using FitConnect.Services.Interfaces;
using FitConnect.Services.Models.v1.Destination;
using Microsoft.Extensions.Logging;

namespace FitConnect.Services;

internal class DestinationService : RestCallService, IDestinationService {
    private const string ApiVersion = "v1";
    private readonly IOAuthService _oAuthService;

    public DestinationService(string baseUrl, IOAuthService oAuthService, string version = "v1",
        ILogger? logger = null) : base(
        $"{baseUrl}/{version}", logger) {
        _oAuthService = oAuthService;
    }

    /// <summary>
    ///     <para>@PostMapping("/destinations")</para>
    /// </summary>
    /// <param name="createDestination">RequestBody</param>
    /// <returns></returns>
    public PrivateDestinationDto CreateDestination(CreateDestinationDto createDestination) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>@GetMapping("/destinations/{destinationId}")</para>
    /// </summary>
    /// <param name="destinationId">PathVariable</param>
    public async Task<PublicDestinationDto?> GetDestinationAsync(Guid destinationId) {
        await _oAuthService.EnsureAuthenticatedAsync();
        var result =
            await RestCall<PublicDestinationDto>($"/destinations/{destinationId}", HttpMethod.Get);
        return result;
    }


    /// <summary>
    ///     <para>@DeleteMapping("/destinations/{destinationId}")</para>
    /// </summary>
    /// <param name="destinationId">PathVariable</param>
    /// <returns></returns>
    public bool DeleteDestination(string destinationId) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>@PatchMapping("/destinations/{destinationId}")</para>
    /// </summary>
    /// <param name="destinationId">PathVariable</param>
    /// <param name="patchDestination">RequestBody</param>
    /// <returns></returns>
    public PrivateDestinationDto PatchDestination(string destinationId,
        PatchDestinationDto patchDestination) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>@GetMapping("/destinations")</para>
    /// </summary>
    /// <param name="offset">How many entries of the result should be skipped</param>
    /// <param name="limit">How many result sets should be loaded</param>
    /// <returns></returns>
    public DestinationListDto ListDestinations(int offset, int limit) {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     <para>@PutMapping("/destinations/{destinationId}")</para>
    /// </summary>
    /// <param name="destinationId">PathVariable</param>
    /// <param name="updateDestination">RequestBody</param>
    /// <returns></returns>
    public PrivateDestinationDto UpdateDestination(string destinationId,
        UpdateDestinationDto updateDestination) {
        throw new NotImplementedException();
    }

    public async Task<string> GetPublicKey(Guid destinationId) {
        await _oAuthService.EnsureAuthenticatedAsync();
        var destinationInfo =
            await RestCall<PublicDestinationDto>($"/destinations/{destinationId}",
                HttpMethod.Get);

        if (destinationInfo?.EncryptionKid == null)
            throw new ArgumentException("EncryptionKid was not received");

        var keyId = destinationInfo.EncryptionKid;

        // /v1/destinations/{{destinationId}}/keys/{{keyId}
        var result = await RestCallForString($"/destinations/{destinationId}/keys/{keyId}",
            HttpMethod.Get);

        return result;
    }
}
